# This script is based on the "ChangeAtZ" script. 

# This script Allows you to enter the M106 S value where your fan actually starts turning 
# and the S value where it reaches maximum speed
# it then recalculates the 0-100% fan based on this range and rounds to the nearest whole number to avoid issues with RRF scale 0.0-1.0

#V1.0 1/17/2022 Initial version

from ..Script import Script
#from UM.Logger import Logger
import re

class RRFFanScale(Script):
    version = "5.1.1"
    def __init__(self):
        super().__init__()

    def getSettingDataString(self):
        return """{
            "name":"RRFFanScale",
            "key":"RRFFanScale",
            "metadata": {},
            "version": 2,
            "settings":
            {
                "lowfanSpeed":
                {
                    "label": "Low Fan Speed",
                    "description": "Low Fan Speed (0-255) M106 Sx (Fan Starts Spinning)",
                    "unit": "S value",
                    "type": "int",
                    "default_value": 0,
                    "minimum_value": "0",
                    "minimum_value_warning": "0",
                    "maximum_value_warning": "127"
                },
                "highfanSpeed":
                {
                    "label": "High Fan Speed",
                    "description": "High Fan Speed (0-255) M106 Sxxx (Fan Max Spinning)",
                    "unit": "S value",
                    "type": "int",
                    "default_value": 255,
                    "minimum_value": "0",
                    "minimum_value_warning": "128",
                    "maximum_value_warning": "255"
                }
            }
        }"""

    def getValue(self, line, key, default = None): #replace default getvalue due to comment-reading feature
        if not key in line or (";" in line and line.find(key) > line.find(";") and
                                   not ";ChangeAtZ" in key and not ";LAYER:" in key):
            return default
        subPart = line[line.find(key) + len(key):] #allows for string lengths larger than 1
        if ";ChangeAtZ" in key:
            m = re.search("^[0-4]", subPart)
        elif ";LAYER:" in key:
            m = re.search("^[+-]?[0-9]*", subPart)
        else:
            #the minus at the beginning allows for negative values, e.g. for delta printers
            m = re.search("^[-]?[0-9]*\.?[0-9]*", subPart)
        if m == None:
            return default
        try:
            return float(m.group(0))
        except:
            return default
        
    def execute(self, data):
      LowFanScalespeed = int(self.getSettingValueByKey("lowfanSpeed"))
      HighFanScalespeed = int(self.getSettingValueByKey("highfanSpeed"))
      fanrange = HighFanScalespeed - LowFanScalespeed
      fanunits = fanrange / 100
      layer = 0              
      index = 0
      oldpercent = 0
      oldfanspeed = 0
      newfanspeed = 0
      for active_layer in data:
        modified_gcode = ""
        lines = active_layer.split("\n")

        for line in lines:
          if "M106" in line:
            oldfanspeed = self.getValue(line, "S", oldfanspeed) 
            oldpercent = (oldfanspeed / 255 ) * 100
            newfanspeed = round((fanunits * oldpercent) + LowFanScalespeed)
            if newfanspeed < 1: 
              newfanspeed = 0
            modified_gcode += "M106 S%d\n" % newfanspeed
          elif "M107" in line:
            modified_gcode += "M106 S0\n"

          else:         
            modified_gcode += line + "\n"

          if ";LAYER:" in line: 
            layer = self.getValue(line, ";LAYER:", layer)

        data[index] = modified_gcode
        index += 1
        
      return data
